﻿init python:
    from copy import copy
    import random
    import time
    random.seed(time.time())
    def distance(a, b):
        if a >= b:
            result = a - b
        else:
            result = b - a
        return result
    class cosmic_orb(renpy.Displayable):
        def __init__(self, name, image):
            super(renpy.Displayable,self).__init__()
            self.name = name
            self.image = image
            self.x = 0
            self.y = 0

            self.x_target = 0
            self.y_target = 0
            self.parent = None
            self.visible = True
            self.highlighted = False
            self.speed = random.random()*4

        def render(self,width,height,st,at):
            if self.visible:
                if self.y < self.y_target:
                    self.y += 6 + self.speed
                if self.y > self.y_target:
                    self.y -= 6 + self.speed
                if self.x < self.x_target:
                    self.x += 6
                if self.highlighted:
                    t = Transform(self.image, alpha = .4)
                else:
                    t = Transform(self.image)

                child_render = renpy.render(t, width, height, st, at)
                cw, ch = child_render.get_size()

                rv = renpy.Render(cw, ch)
                rv.blit(child_render, (self.x, self.y))

                renpy.redraw(self, 0)
                return rv
            else:
                return renpy.Render(0, 0)
        def event(self, ev, x, y, st):
            # if renpy.map_event(ev, "mousedown_1"):
            # if ev.type == 1025:
            if not self.parent.dragging and renpy.map_event(ev, "mousedown_1"):
                self.parent.mouse_down()
                raise renpy.IgnoreEvent()
            elif renpy.map_event(ev, "mouseup_1"):
                self.parent.mouse_up()
                
                raise renpy.IgnoreEvent()
            
            if self.parent.dragging:
                if distance(y, self.y+64) < 50:
                    if distance(x, self.x+64) < 50:
                        renpy.notify(ev.type)
                        self.highlighted = True
                        raise renpy.IgnoreEvent()

    class match_manager:
        def __init__(self, orbs, x, y):
            self.orbs = orbs
            self.x = x
            self.y = y
            self.start_x = 1920//2 + (-self.x*64)
            self.start_y = 1080//2 + (self.y*64)
            self.field = self.fill(x,y)
            self.dragging = False
            self.tries = 0
        def Shuffle(self):
            for column in self.field:
                random.shuffle(column)
            self.fall()
        def mouse_down(self):
            self.dragging = True
        def mouse_up(self):
            self.tries += 1
            self.dragging = False
            for n,column in enumerate(self.field):
                self.field[n] = [i for i in column if not i.highlighted]
            
            renpy.restart_interaction()
            self.top_off()
            self.fall()
        def remove(self, orb):
            for column in self.field:
                if orb in column:
                    column.remove(orb)
                    renpy.restart_interaction()
                    self.top_off()
                    self.fall()
                    return
        def fall(self):
            for column in self.field:
                for y,i in enumerate(column):
                    i.y_target = -y*128 + self.start_y -128
        def top_off(self):
            for x,column in enumerate(self.field):
                count = len(column)
                for i in range(count, self.y):
                    orb = copy(random.choice(self.orbs))
                    orb.parent = self
                    orb.x_target = x*128 + self.start_x
                    orb.x = x*128 + self.start_x
                    orb.y = -(1+self.y)*128 + self.start_y -128
                    orb.y_target = -(1+self.y)*128 + self.start_y -128
                    column.append(orb)

        def fill(self, width, height):
            field = []
            for x in range(width):
                column = []
                for y in range(height):
                    orb = copy(random.choice(self.orbs))
                    orb.parent = self
                    orb.x_target = (x*128) + self.start_x
                    orb.x = (x*128) + self.start_x
                    orb.y_target = -(y*128) + self.start_y -128
                    column.append(orb)
                field.append(column)
            return field

init:
    image orb_1:
        Crop((0,0,32,32), "match_orbs/sprite.png")
        zoom 4 nearest True
    image orb_2:
        Crop((32,0,32,32), "match_orbs/sprite.png")
        zoom 4 nearest True
    image orb_3:
        Crop((64,0,32,32), "match_orbs/sprite.png")
        zoom 4 nearest True
    image orb_4:
        Crop((96,0,32,32), "match_orbs/sprite.png")
        zoom 4 nearest True
    image orb_5:
        Crop((0,32,32,32), "match_orbs/sprite.png")
        zoom 4 nearest True
    image orb_6:
        Crop((32,32,32,32), "match_orbs/sprite.png")
        zoom 4 nearest True
    image orb_7:
        Crop((64,32,32,32), "match_orbs/sprite.png")
        zoom 4 nearest True
    image orb_8:
        Crop((96,32,32,32), "match_orbs/sprite.png")
        zoom 4 nearest True
    image orb_9:
        Crop((0,64,32,32), "match_orbs/sprite.png")
        zoom 4 nearest True

screen match_orbs_screen():
    modal True
    default g = match_manager(
        [
            cosmic_orb("1", "orb_1"),
            cosmic_orb("2", "orb_2"),
            cosmic_orb("3", "orb_3"),
            cosmic_orb("4", "orb_4"),
            cosmic_orb("5", "orb_5"),
            cosmic_orb("6", "orb_6"),
            cosmic_orb("7", "orb_7"),
            cosmic_orb("8", "orb_8"),
            cosmic_orb("9", "orb_9"),
        ], 6, 5
        )
    frame:
        align .5,.5 padding 20,20 background "#23004f75"
        frame:
            xysize g.x*32, g.y*32 padding 0,0 background None
            at zoom(4, True)

    for y in g.field:
        for x in y:
            add x
    vbox:
        button:
            text "Shuffle"
            action Function(g.Shuffle)
        text str(g.tries)


label match_orbs_example:
    call screen match_orbs_screen
